package org.processmining.plugins.plg.generator.log;

import java.io.IOException;
import java.util.Iterator;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.events.Logger.MessageLevel;
import org.processmining.plugins.plg.config.BasicPluginConfiguration;
import org.processmining.plugins.plg.gui.LogGeneratorParameters;
import org.processmining.plugins.plg.models.PLGLog;
import org.processmining.plugins.plg.models.PLGLogCollection;
import org.processmining.plugins.plg.models.PLGProcessCollection;
import org.processmining.plugins.plg.models.PlgProcess;

/**
 * Class for the generation of random business process logs
 * 
 * @author Andrea Burattin
 */
public class PLGLogGenerator {

	/**
	 * This method implements the plugin for the execution and the generation of
	 * a log, given a configuration and a process model
	 * 
	 * @param context
	 * @param process the process model
	 * @param configuration the new log configuration
	 * @return a log
	 */
	@Plugin (
		name = "PLG Log generator",
		parameterLabels = { "Process model", "Configuration" },
		returnLabels = { "Log" },
		returnTypes = { XLog.class },
		userAccessible = false,
		help = "Log generator based on the PLG framework."
	)
	@UITopiaVariant (
		affiliation = BasicPluginConfiguration.AFFILIATION,
		author = BasicPluginConfiguration.AUTHOR,
		email = BasicPluginConfiguration.EMAIL,
		pack = "PLG - Processes and Log Generator"
	)
	public XLog generateLog(PluginContext context, PlgProcess process, PLGLogGeneratorConfiguration configuration) {
		try {
			return process.getProcess().generateXESLog(
					configuration.getNumberOfTraces(),
					configuration.getPercentOfInterval(),
					configuration.getPercentOfNoise());
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	
	/**
	 * This method implements the plugin for the execution and the generation of
	 * a set of logs, given a set of processes
	 * 
	 * @param context
	 * @param processes the set of process models
	 * @return a collection of logs
	 */
	@Plugin (
		name = "Generate a collection of logs with PLG",
		parameterLabels = { "Process models" },
		returnLabels = { "Logs collection" },
		returnTypes = { PLGLogCollection.class },
		userAccessible = true,
		help = "Collection of logs generator based on the PLG framework."
	)
	@UITopiaVariant (
		affiliation = BasicPluginConfiguration.AFFILIATION,
		author = BasicPluginConfiguration.AUTHOR,
		email = BasicPluginConfiguration.EMAIL,
		pack = "PLG - Processes and Log Generator"
	)
	public PLGLogCollection generateLogs(UIPluginContext context, PLGProcessCollection processes) {
		
		LogGeneratorParameters parameters = new LogGeneratorParameters();
		InteractionResult result = context.showConfiguration("Logs generator parameters", parameters);
		if (result.equals(InteractionResult.CANCEL)) {
			context.log("Collection configuration canceled", MessageLevel.DEBUG);
			context.getFutureResult(0).cancel(true);
		}
		PLGLogCollectionGeneratorConfiguration settings = parameters.getSettings();
		PLGLogCollection collection = new PLGLogCollection(settings.getLogNo());
		collection.setConfiguration(settings);
		
		context.log("Collection configuration complete", MessageLevel.DEBUG);
		
		context.getFutureResult(0).setLabel(settings.getName());
		context.getProgress().setMinimum(0);
		context.getProgress().setMaximum(settings.getLogNo() * settings.getSingleLog().getNumberOfTraces());
		context.getProgress().setCaption("Generating processes...");
		context.getProgress().setIndeterminate(false);
		
		context.log("Context configuration complete", MessageLevel.DEBUG);

		PLGLog l = null;
		String logName = null;

		for (Iterator<PlgProcess> iter = processes.iterator(); iter.hasNext();) {
			PlgProcess p = iter.next();
			for (int i = 0; i < settings.getLogNo(); i++) {
				logName = "Log " + (i+1);
				l = new PLGLog(generateLog(context, p, settings.getSingleLog()), logName);
				collection.put(p, l);
				context.getProgress().inc();
			}
		}

		context.log("Processes generation complete", MessageLevel.DEBUG);
		
		return collection;
	}
}
