package org.processmining.plugins.plg.generator.process;


import java.util.concurrent.CancellationException;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.events.Logger.MessageLevel;
import org.processmining.plugins.plg.config.BasicPluginConfiguration;
import org.processmining.plugins.plg.gui.ProcessGeneratorParameters;
import org.processmining.plugins.plg.logic.models.Process;
import org.processmining.plugins.plg.models.PLGProcessCollection;
import org.processmining.plugins.plg.models.PlgProcess;

/**
 * Class for the generation of random business process
 * 
 * @author Andrea Burattin
 */
public class PLGProcessGenerator {
	
	/**
	 * This method implements the generation of a single process, starting from
	 * the given configuration
	 * 
	 * @param context
	 * @param configuration the new process configuration
	 * @return the generated process
	 */
	@Plugin (
		name = "PLG Process generator",
		parameterLabels = { "Configuration" },
		returnLabels = { "Process model" },
		returnTypes = { PlgProcess.class },
		userAccessible = false,
		help = "Process generator based on the PLG framework."
	)
	@UITopiaVariant (
		affiliation = BasicPluginConfiguration.AFFILIATION,
		author = BasicPluginConfiguration.AUTHOR,
		email = BasicPluginConfiguration.EMAIL,
		pack = "PLG - Processes and Log Generator"
	)
	public PlgProcess generateProcess(PluginContext context, PLGProcessGeneratorConfiguration configuration) {
		PlgProcess process = new PlgProcess();
		Process p = new Process("Test process");
		context.log("Process randomization...", MessageLevel.DEBUG);
		p.randomize(configuration);
		process.setProcess(p);
		context.log("Process generation complete", MessageLevel.DEBUG);
		return process;
	}
	
	
	/**
	 * This method implements the generator of a collection of processes
	 * 
	 * @param context
	 * @param configuration the configuration of the collection of processes
	 * @return the collection of processes
	 */
	@Plugin (
		name = "Generate a collection of processes with PLG",
		parameterLabels = { },
		returnLabels = { "Process model collection"},
		returnTypes = { PLGProcessCollection.class },
		userAccessible = true,
		help = "Collection of process generator based on the PLG framework."
	)
	@UITopiaVariant (
		affiliation = BasicPluginConfiguration.AFFILIATION,
		author = BasicPluginConfiguration.AUTHOR,
		email = BasicPluginConfiguration.EMAIL,
		pack = "PLG - Processes and Log Generator"
	)
	public PLGProcessCollection generateProcesses(UIPluginContext context) {
		
		ProcessGeneratorParameters parameters = new ProcessGeneratorParameters();
		InteractionResult result = context.showConfiguration("Process generator parameters", parameters);
		if (result.equals(InteractionResult.CANCEL)) {
			context.log("Collection configuration canceled", MessageLevel.DEBUG);
			context.getFutureResult(0).cancel(true);
		}
		PLGProcessCollectionGeneratorConfiguration settings = parameters.getSettings();
		PLGProcessCollection collection = new PLGProcessCollection(settings.getInstancesNo());
		collection.setConfiguration(settings);
		
		context.log("Collection configuration complete", MessageLevel.DEBUG);
		
		context.getFutureResult(0).setLabel(settings.getCollectionName());
		context.getProgress().setMinimum(0);
		context.getProgress().setMaximum(settings.getInstancesNo());
		context.getProgress().setCaption("Generating processes...");
		context.getProgress().setIndeterminate(false);
		
		context.log("Context configuration complete", MessageLevel.DEBUG);
		
		try {
			PlgProcess p = null;
			
			for (int i = 0; i < settings.getInstancesNo(); i++) {
				/* Better performances by directly calling the method. Since
				 * it's in this class too, I assume I know it.
				 */
//				p = pc.tryToFindOrConstructFirstObject(PLGProcess.class, null, null, settings.getSingleProcess());
				p = generateProcess(context, settings.getSingleProcess());
				p.setName("Process " + (i + 1));
				collection.add(p);
				context.getProgress().inc();
			}
			
			context.log("Processes generation complete", MessageLevel.DEBUG);
			
		} catch (CancellationException e) {
			e.printStackTrace();
		}

		return collection;
	}
}
