package org.processmining.plugins.plg.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.plaf.basic.BasicTreeUI;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import org.deckfour.uitopia.ui.util.ImageLoader;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.providedobjects.ProvidedObjectManager;
import org.processmining.plugins.plg.models.PLGLog;
import org.processmining.plugins.plg.models.PLGLogCollection;
import org.processmining.plugins.plg.models.PlgProcess;

import com.fluxicon.slickerbox.components.HeaderBar;
import com.fluxicon.slickerbox.factory.SlickerDecorator;
import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * This class contains the logs collection viewer
 * 
 * @author Andrea Burattin
 */
@Plugin(
	name = "Log collection visualization", 
	returnLabels = { "Visualized log collection" }, 
	returnTypes = { JComponent.class }, 
	parameterLabels = { "Log collection" }, 
	userAccessible = true
)
@Visualizer
public class LogCollectionViewer {
	
	/**
	 * This class is a listener for the list of processes and logs. Its role is
	 * to show the correct process or log into the proper panel, when it is
	 * clicked on the list.
	 */
	private class LogListListener implements ActionListener {
		JPanel containerProcess;
		JTree processesLogList;
		UIPluginContext context;
		JLabel messages;
		
		
		/**
		 * Basic constructor
		 * 
		 * @param containerProcess the JPanel with the visualizer
		 * @param processesLogList the component with the list
		 * @param context the visualizer context
		 */
		public LogListListener(JLabel messages, JPanel containerProcess, JTree processesLogList, UIPluginContext context) {
			super();
			this.messages = messages;
			this.processesLogList = processesLogList;
			this.containerProcess = containerProcess;
			this.context = context;
		}
		
		
		@Override
		public void actionPerformed(ActionEvent arg0) {
			if (processesLogList.getSelectionCount() == 0) {
				return;
			}
			
			TreePath path = processesLogList.getSelectionPath();
			Object o = ((DefaultMutableTreeNode)path.getLastPathComponent()).getUserObject();
			
			JComponent visualizer = null;
			String title = null;
			
			try {
				
				if (o instanceof PlgProcess) {
					final PlgProcess p = (PlgProcess)o;
					
					messages.setText("Showing " + p.getName() + "... ");

					visualizer = context.tryToFindOrConstructFirstObject(JComponent.class, null, null, p);
					title = p.getName();

					messages.setText(messages.getText() + "done!");
					
					
				} else if (o instanceof PLGLog) {
					
					PlgProcess p = (PlgProcess) ((DefaultMutableTreeNode)path.getPathComponent(1)).getUserObject();
					PLGLog l = (PLGLog)o;
					XLog log = l.getLog();
					
					messages.setText("Showing " + l.getId() + "... ");
					
//					visualizer = context.tryToFindOrConstructFirstNamedObject(JComponent.class, "Log Summary", null, null, log);
					visualizer = context.tryToFindOrConstructFirstObject(JComponent.class, null, null, log);
					visualizer.setBackground(Color.gray);
					title = p.getName() + " -> " + l.getId();
					
					messages.setText(messages.getText() + "done!");
					
				} else {
					return;
				}
			
			} catch (ConnectionCannotBeObtained e) {
				e.printStackTrace();
			}
			
			containerProcess.removeAll();
			containerProcess.add(GUIUtils.prepareTitle(title), BorderLayout.NORTH);
			containerProcess.add(visualizer, BorderLayout.CENTER);
			containerProcess.validate();

			// cleaning the jcomponent of the visualizer plugin from the workspace
			context.getGlobalContext().getResourceManager().getResourceForInstance(visualizer).destroy();
		}
	}
	
	
	/**
	 * This class is a listener for using a log
	 */
	private class LogUseListener implements ActionListener {
		JTree processesLogList;
		UIPluginContext context;
		JLabel messages;
		
		
		/**
		 * Basic constructor
		 * 
		 * @param processesLogList the component with the list
		 * @param context the visualizer context
		 */
		public LogUseListener(JLabel messages, JTree processesLogList, UIPluginContext context) {
			super();
			this.messages = messages;
			this.processesLogList = processesLogList;
			this.context = context;
		}
		
		
		@Override
		public void actionPerformed(ActionEvent arg0) {
			TreePath path = processesLogList.getSelectionPath();
			DefaultMutableTreeNode tn = (DefaultMutableTreeNode)path.getLastPathComponent();
			Object o = tn.getUserObject();
			if (o instanceof PLGLog) {
				PLGLog l = (PLGLog)o;
				
				messages.setText("Exporting " + l.getId() + " to workspace... ");
				
				PlgProcess p = (PlgProcess) ((DefaultMutableTreeNode)path.getPathComponent(1)).getUserObject();
				ProvidedObjectManager pom = context.getProvidedObjectManager();
				String name = l.getId() + " (from " + p.getName() + ")";
				
				// add the log to the workspace
				pom.createProvidedObject(name, l.getLog(), XLog.class, context);
				context.getGlobalContext().getResourceManager().getResourceForInstance(l.getLog()).setFavorite(true);
				
				messages.setText(messages.getText() + "done!");
			}
		}
	}
	

	@PluginVariant(
		requiredParameterLabels = {0}
	)
	public JComponent visualize(UIPluginContext context, PLGLogCollection collection) {
		JPanel containerMain = SlickerFactory.instance().createGradientPanel(new Color(60,60,60), new Color(20,20,20));
		JPanel containerBody = new JPanel();
		JPanel containerBodyLeft = new JPanel();
		JPanel containerBodyCenter = new JPanel();
		JPanel containerProperties = SlickerFactory.instance().createRoundedPanel(15, Color.gray);
		JPanel containerList = SlickerFactory.instance().createRoundedPanel(15, Color.gray);
		JButton buttonReloadView = SlickerFactory.instance().createButton("Show");
		final JButton buttonUseLog = SlickerFactory.instance().createButton("Export to workspace");
		JPanel containerProcess = SlickerFactory.instance().createRoundedPanel(15, Color.gray);
		HeaderBar title = new HeaderBar("Basic collection properties");
		final JTree processesLogList;
		final JLabel messages = GUIUtils.prepareLabel(" ");
		JPanel messageAndCreditPanel = new JPanel(new BorderLayout());
		messageAndCreditPanel.setOpaque(false);
		
		messages.setBorder(new EmptyBorder(0, 0, 0, 0));
		messages.setOpaque(false);
		messages.setForeground(Color.gray);
		messages.setHorizontalAlignment(SwingConstants.RIGHT);
		
		containerProperties.setLayout(new GridBagLayout());
		containerList.setLayout(new BorderLayout());
		containerProcess.setLayout(new BorderLayout());
		
		containerBody.setOpaque(false);
		containerBodyLeft.setOpaque(false);
		containerBodyLeft.setLayout(new BorderLayout());
		((BorderLayout)containerBodyLeft.getLayout()).setVgap(5);
		containerBodyLeft.setBorder(new EmptyBorder(25, 25, 25, 25));
		
		containerBodyCenter.setOpaque(false);
		containerBodyCenter.setLayout(new BorderLayout());
		containerBodyCenter.setBorder(new EmptyBorder(25, 0, 25, 25));
		
		containerBody.setLayout(new BorderLayout());
		containerBody.add(containerBodyLeft, BorderLayout.WEST);
		containerBody.add(containerBodyCenter, BorderLayout.CENTER);
		
		containerBodyLeft.add(containerProperties, BorderLayout.NORTH);
		containerBodyLeft.add(containerList, BorderLayout.CENTER);
		containerBodyLeft.add(messageAndCreditPanel, BorderLayout.SOUTH);
		
		messageAndCreditPanel.add(messages, BorderLayout.NORTH);
		messageAndCreditPanel.add(GUIUtils.credits, BorderLayout.SOUTH);
		
		containerBodyCenter.add(containerProcess);
		
		// collection properties
		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.EAST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 0; c.gridy = 0;
		containerProperties.add(GUIUtils.prepareLabel("Logs per process: ", SwingConstants.RIGHT), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 1; c.gridy = 0;
		containerProperties.add(GUIUtils.prepareLabel(collection.getConfiguration().getLogNo()), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.EAST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 0; c.gridy = 1;
		containerProperties.add(GUIUtils.prepareLabel("Number of cases: ", SwingConstants.RIGHT), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 1; c.gridy = 1;
		containerProperties.add(GUIUtils.prepareLabel(collection.getConfiguration().getSingleLog().getNumberOfTraces()), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.EAST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 0; c.gridy = 2;
		containerProperties.add(GUIUtils.prepareLabel("Percent of noise: ", SwingConstants.RIGHT), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 1; c.gridy = 2;
		containerProperties.add(GUIUtils.prepareLabel(collection.getConfiguration().getSingleLog().getPercentOfNoise()), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.EAST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 0; c.gridy = 3;
		containerProperties.add(GUIUtils.prepareLabel("Percent of activity as interval: ", SwingConstants.RIGHT), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 1; c.gridy = 3;
		containerProperties.add(GUIUtils.prepareLabel(collection.getConfiguration().getSingleLog().getPercentOfInterval()), c);
		
		// list of logs and processes
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(collection.getConfiguration().getName());
		DefaultTreeModel processesListModel = new DefaultTreeModel(root);
		for (PlgProcess p : collection.keySet()) {
			DefaultMutableTreeNode nodeProcess = new DefaultMutableTreeNode(p);
			root.add(nodeProcess);
			for (PLGLog l : collection.get(p)) {
				DefaultMutableTreeNode nodeLog = new DefaultMutableTreeNode(l);
				nodeProcess.add(nodeLog);
			}
		}
		
		UIManager.put("Tree.hash", Color.darkGray);
		
		processesLogList = new JTree(processesListModel);

		((BasicTreeUI)processesLogList.getUI()).setExpandedIcon(null);
		((BasicTreeUI)processesLogList.getUI()).setCollapsedIcon(null);
		
		processesLogList.setForeground(Color.white);
		processesLogList.setBackground(Color.gray);
		processesLogList.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		processesLogList.expandRow(1);
		
		processesLogList.addTreeSelectionListener(new TreeSelectionListener() {
		    public void valueChanged(TreeSelectionEvent e) {
		        DefaultMutableTreeNode node = (DefaultMutableTreeNode)processesLogList.getLastSelectedPathComponent();
		        Object nodeInfo = node.getUserObject();
		        buttonUseLog.setEnabled((nodeInfo instanceof PLGLog));
		        messages.setText(" ");
		    }
		});
		
		DefaultTreeCellRenderer renderer = (DefaultTreeCellRenderer) processesLogList.getCellRenderer();
		renderer.setBackground(Color.gray);
		renderer.setBackgroundNonSelectionColor(Color.gray);
		renderer.setBackgroundSelectionColor(Color.darkGray);
		renderer.setForeground(Color.WHITE);
		renderer.setTextNonSelectionColor(Color.WHITE);
		renderer.setTextSelectionColor(Color.white);
		renderer.setOpenIcon(getProcessImageIcon());
		renderer.setClosedIcon(getProcessImageIcon());
		renderer.setLeafIcon(getLogImageIcon());
		
		JScrollPane processesScrollPane = new JScrollPane(processesLogList);
		SlickerDecorator.instance().decorate(processesScrollPane, Color.gray, Color.black, Color.darkGray);
		processesScrollPane.setBorder(new EmptyBorder(0, 0, 0, 0));
		buttonReloadView.addActionListener(new LogListListener(messages, containerProcess, processesLogList, context));
		buttonUseLog.addActionListener(new LogUseListener(messages, processesLogList, context));
		
		JPanel tmp0 = new JPanel(); tmp0.setOpaque(false);
		tmp0.setLayout(new BorderLayout());
		tmp0.setBorder(new EmptyBorder(5, 0, 0, 0));
		tmp0.add(buttonReloadView, BorderLayout.CENTER);
		tmp0.add(buttonUseLog, BorderLayout.EAST);
		buttonUseLog.setEnabled(false);
		
		JLabel tmp1 = GUIUtils.prepareLabel("Double click a process to show its logs:");
		tmp1.setBorder(new EmptyBorder(5, 5, 10, 5));
		containerList.add(tmp1, BorderLayout.NORTH);
		containerList.add(processesScrollPane, BorderLayout.CENTER);
		containerList.add(tmp0, BorderLayout.SOUTH);
		
		// final layout
		
		containerMain.setLayout(new BorderLayout());
		containerMain.add(title, BorderLayout.NORTH);
		containerMain.add(containerBody, BorderLayout.CENTER);
		
		return containerMain;
	}
	
	
	/**
	 * This method returns the correctly scaled image icon for a log
	 *  
	 * @return the image icon for the log
	 */
	private static ImageIcon getLogImageIcon() {
		ImageIcon iconLog = new ImageIcon(ImageLoader.load("resourcetype_log_30x35.png"));
		Image img = iconLog.getImage();  
		Image newimg = img.getScaledInstance(15, 17, java.awt.Image.SCALE_SMOOTH);  
		return new ImageIcon(newimg);
	}
	
	
	/**
	 * This method returns the correctly scaled image icon for a process model 
	 * 
	 * @return the image icon for the process model
	 */
	private static ImageIcon getProcessImageIcon() {
		ImageIcon iconLog = new ImageIcon(ImageLoader.load("resourcetype_model_30x35.png"));
		Image img = iconLog.getImage();  
		Image newimg = img.getScaledInstance(15, 17, java.awt.Image.SCALE_SMOOTH);  
		return new ImageIcon(newimg);
	}
	
}
