package org.processmining.plugins.plg.miner;

import org.deckfour.xes.classification.XEventNameClassifier;
import org.deckfour.xes.info.XLogInfoFactory;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.events.Logger.MessageLevel;
import org.processmining.models.heuristics.HeuristicsNet;
import org.processmining.plugins.heuristicsnet.miner.heuristics.miner.HeuristicsMiner;
import org.processmining.plugins.heuristicsnet.miner.heuristics.miner.settings.HeuristicsMinerSettings;
import org.processmining.plugins.plg.config.BasicPluginConfiguration;
import org.processmining.plugins.plg.models.PLGLog;
import org.processmining.plugins.plg.models.PLGLogCollection;
import org.processmining.plugins.plg.models.PLGMineResultCollection;
import org.processmining.plugins.plg.models.PlgProcess;

/**
 * This class contains a set of wrapper for some well known process mining
 * plugins
 * 
 * @author Andrea Burattin
 */
public class PLGMiner {

	@Plugin (
		name = "Mine a collection of logs with Heuristics Miner (PLG)",
		parameterLabels = { "Logs collection" },
		returnLabels = { "Mine collection" },
		returnTypes = { PLGMineResultCollection.class },
		userAccessible = true,
		help = "Heuristics Miner wrapper for a collection of logs."
	)
	@UITopiaVariant (
		affiliation = BasicPluginConfiguration.AFFILIATION,
		author = BasicPluginConfiguration.AUTHOR,
		email = BasicPluginConfiguration.EMAIL,
		pack = "PLG - Processes and Log Generator"
	)
	public PLGMineResultCollection wrapHeuristicsMiner(UIPluginContext context, PLGLogCollection collection) {
		PLGMineResultCollection resultsCollection = new PLGMineResultCollection("Heuristics Miner on " + collection.getConfiguration().getName());
		HeuristicsNet hn;
		
		context.getFutureResult(0).setLabel(resultsCollection.getName());
		context.getProgress().setMinimum(0);
		context.getProgress().setMaximum(collection.getConfiguration().getLogNo() * collection.size());
		context.getProgress().setIndeterminate(false);
		
		context.log("Starting mining...", MessageLevel.DEBUG);
		for (PlgProcess p : collection.keySet()) {
			for (PLGLog l : collection.get(p)) {
//				try {
					HeuristicsMinerSettings hms = new HeuristicsMinerSettings();
					hms.setClassifier(new XEventNameClassifier());
					HeuristicsMiner hm = new HeuristicsMiner(context, l.getLog(), XLogInfoFactory.createLogInfo(l.getLog()), hms);
					hn = hm.mine();
//					hn = context.tryToFindOrConstructFirstNamedObject(HeuristicsNet.class, "Mine for a Heuristics Net using Heuristics Miner", null, null, l.getLog());
					resultsCollection.put(p, l, hn);
//				} catch (ConnectionCannotBeObtained e) {
//					e.printStackTrace();
//				}
				context.getProgress().inc();
			}
		}
		context.log("Mining complete", MessageLevel.DEBUG);

		return resultsCollection;
	}
	
	
	/*@SuppressWarnings("unchecked")
	@Plugin (
		name = "PLG Fuzzy Miner wrapper",
		parameterLabels = { "Logs collection" },
		returnLabels = { "Mine collection" },
		returnTypes = { PLGMineResultCollection.class },
		userAccessible = true,
		help = "Fuzzy Miner wrapper for a collection of logs."
	)
	@UITopiaVariant (
		affiliation = BasicPluginConfiguration.AFFILIATION,
		author = BasicPluginConfiguration.AUTHOR,
		email = BasicPluginConfiguration.EMAIL,
		pack = "PLG - Processes and Log Generator"
	)
	public PLGMineResultCollection wrapFuzzyMiner(UIPluginContext context, PLGLogCollection collection) {
		PLGMineResultCollection resultsCollection = new PLGMineResultCollection("Fuzzy Miner on " + collection.getConfiguration().getName());
		Object fg;
		
		context.getFutureResult(0).setLabel(resultsCollection.getName());
		context.getProgress().setMinimum(0);
		context.getProgress().setMaximum(collection.getConfiguration().getLogNo() * collection.size());
		context.getProgress().setIndeterminate(false);
		
		Class c = null;
		try {
			c = Class.forName("org.processmining.models.graphbased.directed.fuzzymodel.MutableFuzzyGraph");
		} catch (ClassNotFoundException e1) {
			e1.printStackTrace();
			return null;
		}
		
		context.log("Starting mining...", MessageLevel.DEBUG);
		for (PLGProcess p : collection.keySet()) {
			for (PLGLog l : collection.get(p)) {
				try {
					fg = context.tryToFindOrConstructFirstNamedObject(c, "Fuzzy Miner", null, null, l.getLog());
					resultsCollection.put(p, l, fg);
				} catch (ConnectionCannotBeObtained e) {
					e.printStackTrace();
				}
				context.getProgress().inc();
			}
		}
		context.log("Mining complete", MessageLevel.DEBUG);

		return resultsCollection;
	}*/
	
	
	/*@SuppressWarnings("unchecked")
	@Plugin (
		name = "PLG Genetic Miner wrapper",
		parameterLabels = { "Logs collection" },
		returnLabels = { "Mine collection" },
		returnTypes = { PLGMineResultCollection.class },
		userAccessible = true,
		help = "Genetic Miner wrapper for a collection of logs."
	)
	@UITopiaVariant (
		affiliation = BasicPluginConfiguration.AFFILIATION,
		author = BasicPluginConfiguration.AUTHOR,
		email = BasicPluginConfiguration.EMAIL,
		pack = "PLG - Processes and Log Generator"
	)
	public PLGMineResultCollection wrapGeneticMiner(UIPluginContext context, PLGLogCollection collection) {
		PLGMineResultCollection resultsCollection = new PLGMineResultCollection("Genetic Miner on " + collection.getConfiguration().getName());
		Object hnao;
		
		context.getFutureResult(0).setLabel(resultsCollection.getName());
		context.getProgress().setMinimum(0);
		context.getProgress().setMaximum(collection.getConfiguration().getLogNo() * collection.size());
		context.getProgress().setIndeterminate(false);
		
		Class c = null;
		try {
			c = Class.forName("org.processmining.plugins.heuristicsnet.array.visualization.HeuristicsNetArrayObject");
		} catch (ClassNotFoundException e1) {
			e1.printStackTrace();
			return null;
		}
		
		context.log("Starting mining...", MessageLevel.DEBUG);
		for (PlgProcess p : collection.keySet()) {
			for (PLGLog l : collection.get(p)) {
				try {
					hnao = context.tryToFindOrConstructFirstNamedObject(c, "Genetic Miner", null, null, l.getLog());
					resultsCollection.put(p, l, hnao);
				} catch (ConnectionCannotBeObtained e) {
					e.printStackTrace();
				}
				context.getProgress().inc();
			}
		}
		context.log("Mining complete", MessageLevel.DEBUG);
	
		return resultsCollection;
	}*/
	
	
	/*@Plugin (
		name = "PLG Alpha miner wrapper",
		parameterLabels = { "Logs collection" },
		returnLabels = { "Mine collection" },
		returnTypes = { PLGMineResultCollection.class },
		userAccessible = true,
		help = "Alpha Miner wrapper for a collection of logs."
	)
	@UITopiaVariant (
		affiliation = BasicPluginConfiguration.AFFILIATION,
		author = BasicPluginConfiguration.AUTHOR,
		email = BasicPluginConfiguration.EMAIL,
		pack = "PLG - Processes and Log Generator"
	)
	public PLGMineResultCollection wrapAlphaMiner(UIPluginContext context, PLGLogCollection collection) {
		PLGMineResultCollection resultsCollection = new PLGMineResultCollection("Alpha Miner on " + collection.getConfiguration().getName());
		Petrinet pn;
		
		context.getFutureResult(0).setLabel(resultsCollection.getName());
		context.getProgress().setMinimum(0);
		context.getProgress().setMaximum(collection.getConfiguration().getLogNo() * collection.size());
		context.getProgress().setIndeterminate(false);
		
		context.log("Starting mining...", MessageLevel.DEBUG);
		for (PlgProcess p : collection.keySet()) {
			for (PLGLog l : collection.get(p)) {
				AlphaMiner miner = new AlphaMiner();
				try {
					pn = (Petrinet) context.tryToFindOrConstructFirstNamedObject(Object[].class, "Mine for a Petri Net using Alpha-algorithm", null, null, l.getLog())[0];
					resultsCollection.put(p, l, pn);
				} catch (ConnectionCannotBeObtained e) {
					e.printStackTrace();
				}
				context.getProgress().inc();
			}
		}
		context.log("Mining complete", MessageLevel.DEBUG);

		return resultsCollection;
	}*/
}
