package org.processmining.plugins.plg.models;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.deckfour.spex.SXDocument;
import org.deckfour.spex.SXTag;
import org.processmining.framework.annotations.AuthoredType;
import org.processmining.plugins.plg.config.BasicPluginConfiguration;
import org.processmining.plugins.plg.generator.process.PLGProcessCollectionGeneratorConfiguration;
import org.processmining.plugins.plg.generator.process.PLGProcessGeneratorConfiguration;
import org.processmining.plugins.plg.logic.models.Process;
import org.w3c.dom.DOMException;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * This class describes a collection of process models
 * 
 * @author Andrea Burattin
 */
@AuthoredType(
	typeName = "PLG Process collection",
	affiliation = BasicPluginConfiguration.AFFILIATION,
	author = BasicPluginConfiguration.AUTHOR,
	email = BasicPluginConfiguration.EMAIL
)
public class PLGProcessCollection extends CopyOnWriteArrayList<PlgProcess> {

	private static final long serialVersionUID = -8621350605362988576L;
	// this is configuration object that we can use to keep track
	PLGProcessCollectionGeneratorConfiguration configuration;

	
	/**
	 * New constructor
	 * 
	 * @param size the expected size of the collection
	 */
	public PLGProcessCollection(int size) {
//		super(size);
	}
	
	/**
	 * New constructor
	 * 
	 */
	public PLGProcessCollection() {
	}

	
	/**
	 * Getter of the configuration object
	 * 
	 * @return the configuration object
	 */
	public PLGProcessCollectionGeneratorConfiguration getConfiguration() {
		return configuration;
	}

	
	/**
	 * Setter of the configuration object
	 * 
	 * @param configuration the configuration object
	 */
	public void setConfiguration(
			PLGProcessCollectionGeneratorConfiguration configuration) {
		this.configuration = configuration;
	}
	
	
	/**
	 * Export the given process collection to a plg file
	 * 
	 * @param file the destination file
	 * @throws IOException 
	 */
	public void saveCollection(File file) throws IOException {
		ZipOutputStream out = new ZipOutputStream(new FileOutputStream(file));
		// export processes
		for (PlgProcess p : this) {
			p.getProcess().addProcessToPLGZipFile(out);
		}
		
		// export collection metadata
		File tempFile = File.createTempFile("meta", ".xml");
		tempFile.deleteOnExit();
		SXDocument dom = new SXDocument(tempFile);
		
		dom.addComment("Collection information.");
		SXTag meta = dom.addNode("Collection");
		meta.addComment("Basic metadata information about the collection.");
		meta.addChildNode("InstancesNo").addTextNode("" + getConfiguration().getInstancesNo());
		meta.addChildNode("CollectionName").addTextNode(getConfiguration().getCollectionName());
		meta.addChildNode("CreationDate").addTextNode(getConfiguration().getCreationDate().toString());
		
		meta.addComment("Single process generation configuration.");
		meta.addComment("WARNING: more information are required to build a complete new mode!");
		PLGProcessGeneratorConfiguration c = getConfiguration().getSingleProcess();
		meta.addChildNode("ANDPercent").addTextNode("" + c.getANDPercent());
		meta.addChildNode("Deep").addTextNode("" + c.getDeep());
		meta.addChildNode("LoopPercent").addTextNode("" + c.getLoopPercent());
		meta.addChildNode("SequencePercent").addTextNode("" + c.getSequencePercent());
		meta.addChildNode("SingleActivityPercent").addTextNode("" + c.getSingleActivityPercent());
		meta.addChildNode("XORPercent").addTextNode("" + c.getXORPercent());
		
		dom.close();
		
		// add this process to the zip
		FileInputStream in = new FileInputStream(tempFile.getAbsolutePath());
		out.putNextEntry(new ZipEntry("meta.xml"));
		   
		int len; byte[] buf = new byte[1024];
		while ((len = in.read(buf)) > 0) {
			out.write(buf, 0, len);
		}
		out.closeEntry();
		in.close();
		
        out.close();
	}
	
	/**
	 * 
	 * @param file
	 * @return
	 * @throws IOException 
	 * @throws ParserConfigurationException 
	 * @throws SAXException 
	 * @throws ParseException 
	 * @throws DOMException 
	 */
	public static PLGProcessCollection importCollection(InputStream input) throws IOException, SAXException, ParserConfigurationException, DOMException, ParseException {
		File tempFile = File.createTempFile("process", ".xml");
		tempFile.deleteOnExit();
		
		PLGProcessCollection p = new PLGProcessCollection();
		ZipInputStream zin = new ZipInputStream(input);
		ZipEntry entry;
		while((entry = zin.getNextEntry()) != null) {
			int n; byte[] buf = new byte[1024];
			FileOutputStream fileoutputstream = new FileOutputStream(tempFile);
			while ((n = zin.read(buf, 0, 1024)) > -1) {
				fileoutputstream.write(buf, 0, n);
			}
			fileoutputstream.close();
			zin.closeEntry();
			
			if (entry.getName().equals("meta.xml")) {
				
				// Load meta information of the collection ---------------------
				
				FileInputStream fi = new FileInputStream(tempFile);
				DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
				org.w3c.dom.Document doc;
				dbf.setValidating(false);
				dbf.setIgnoringComments(true);
				dbf.setIgnoringElementContentWhitespace(true);
				doc = dbf.newDocumentBuilder().parse(fi);
				
				SimpleDateFormat sdf = new SimpleDateFormat("EEE MMM dd HH:mm:ss zzz yyyy");
				PLGProcessGeneratorConfiguration pgc = new PLGProcessGeneratorConfiguration(0, null, 0, null, 0, 0, 0, 0, 0, 0, null, null);
				PLGProcessCollectionGeneratorConfiguration plgpcgc = new PLGProcessCollectionGeneratorConfiguration("", null, -1);
				
				Node node = doc.getElementsByTagName("Collection").item(0);
				NodeList nodes = node.getChildNodes();
				for (int i = 0; i < nodes.getLength(); i++) {
					
					String nodeName = nodes.item(i).getNodeName();
					
					if (nodeName.equals("InstancesNo")) {
						plgpcgc.setInstancesNo(Integer.parseInt(nodes.item(i).getTextContent()));
					} else if (nodeName.equals("CollectionName")) {
						plgpcgc.setCollectionName(nodes.item(i).getTextContent());
					} else if (nodeName.equals("CreationDate")) {
						plgpcgc.setCreationDate(sdf.parse(nodes.item(i).getTextContent()));
					} else if (nodeName.equals("ANDPercent")) {
						pgc.setANDPercent(Integer.parseInt(nodes.item(i).getTextContent()));
					} else if (nodeName.equals("Deep")) {
						pgc.setDeep(Integer.parseInt(nodes.item(i).getTextContent()));
					} else if (nodeName.equals("LoopPercent")) {
						pgc.setLoopPercent(Integer.parseInt(nodes.item(i).getTextContent()));
					} else if (nodeName.equals("SequencePercent")) {
						pgc.setSequencePercent(Integer.parseInt(nodes.item(i).getTextContent()));
					} else if (nodeName.equals("SingleActivityPercent")) {
						pgc.setSingleActivityPercent(Integer.parseInt(nodes.item(i).getTextContent()));
					} else if (nodeName.equals("XORPercent")) {
						pgc.setXORPercent(Integer.parseInt(nodes.item(i).getTextContent()));
					}
				}
				
				plgpcgc.setSingleProcess(pgc);
				p.setConfiguration(plgpcgc);
				
			} else {
				
				// Load process information ------------------------------------
				
				Process localP = Process.loadProcessFromXMLFile(tempFile);
				PlgProcess localPlg = new PlgProcess();
				localPlg.setProcess(localP);
				localPlg.setName(localP.getName());
				p.add(localPlg);
				
			}
		}
		
		return p;
	}

}
